<?php

// include_paths.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    $vendorDir . '/pear/auth_sasl',
    $vendorDir . '/pear/pear_exception',
);
