<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'PEAR_Exception' => $vendorDir . '/pear/pear_exception/PEAR/Exception.php',
);
