<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(dirname(__FILE__));
$baseDir = dirname($vendorDir);

return array(
    'Pimple' => array($vendorDir . '/pimple/pimple/src'),
    'Auth' => array($vendorDir . '/pear/auth_sasl'),
);
